<?php

namespace App\Console\Commands;

use Doctrine\DBAL\Exception\DatabaseObjectExistsException;
use Doctrine\ORM\EntityNotFoundException;
use GuzzleHttp\Client;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use TranquilCity\TranquilPavement\Domain\Models\User;
use TranquilCity\TranquilPavement\Infrastructure\Doctrine\Repositories\DoctrineUserRepository;

class CreateUserCommand extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'tp:user:create {email}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Create a new user';

    /**
     * Execute the console command.
     *
     * @return void
     * @throws EntityNotFoundException
     */
    public function handle(DoctrineUserRepository $repository)
    {
        $email = $this->argument('email');

        $password = Str::random();

        $user = new User();
        $user->setEmail($email);
        $user->setToken(Str::random(10));
        $user->setPasswordHash(Hash::make($password));

        try {
            $repository->save($user);
            $this->info("User $email created with password: $password");
        } catch (\Exception $e) {
            $this->error($e->getMessage());
        }

    }
}
