<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use TranquilCity\TranquilPavement\Application\Services\TranquilPlaceService;
use TranquilCity\TranquilPavement\Domain\Models\TranquilPlace;
use TranquilCity\TranquilPavement\Domain\Repositories\TranquilPlaceRepository;

class GeocodeTranquilPlacesCommand extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'tp:geocode:places';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Try to geocode Tranquil Places';

    /**
     * Execute the console command.
     *
     * @param TranquilPlaceRepository $repository
     * @param TranquilPlaceService $placeService
     * @return void
     */
    public function handle(TranquilPlaceRepository $repository, TranquilPlaceService $placeService)
    {
        $places = $repository->findAllNotLocated();
        /** @var TranquilPlace[] $places */
        foreach ($places as $place) {
            $placeService->geocode($place);
            $repository->save($place);
            if ($place->hasGeolocation()) {
                $this->output->writeln('Geocoded ' . $place->getSlug());
            } else {
                $this->output->writeln('Failed to geocode ' . $place->getSlug() . '(' . $place->getCaption() . ')');
            }
        }
    }
}
