<?php

namespace App\Console\Commands;

use Doctrine\ORM\EntityNotFoundException;
use Illuminate\Console\Command;
use League\Csv\Reader;
use TranquilCity\TranquilPavement\Domain\Models\Point;
use TranquilCity\TranquilPavement\Domain\Models\PollutionDataPoint;
use TranquilCity\TranquilPavement\Domain\Repositories\PollutionDataPointRepository;

class ImportPollutionDataCommand extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'tp:import:pollution';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Import pollution data from S3';

    /**
     * Execute the console command.
     *
     * @param PollutionDataPointRepository $pollution
     * @return void
     * @throws EntityNotFoundException
     */
    public function handle(PollutionDataPointRepository $pollution)
    {
        $path = storage_path('app/dots_all.csv');
        $this->output->writeln('Reading ' . $path);
        $csv = Reader::createFromPath($path, 'r');
        $csv->setHeaderOffset(0);
        $header = $csv->getHeader();
        $rows = $csv->getRecords($header);
        $count = 5856429;
        foreach($rows as $i => $row) {
            $point = new PollutionDataPoint();
            $point->setGeolocation(new Point($row['latitude'], $row['longitude']));
            $point->setArriDb($row['arri_db']);
            $point->setNo2($row['no2']);
            $point->setPm25($row['pm25']);
            $pollution->add($point);
            if ($i > 0 && ($i % 100000 === 0)) {
                $pollution->saveAll();
                $percentComplete = ((int) ($i / $count * 1000)) / 10;
                $this->output->writeln($percentComplete . '% complete');
            }
        }
        $pollution->saveAll();
        $this->output->writeln('Complete');

    }
}
