<?php

namespace App\Http\Controllers\Api;

use Abraham\TwitterOAuth\TwitterOAuthException;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use TranquilCity\TranquilPavement\Domain\Models\User;
use TranquilCity\TranquilPavement\Domain\Repositories\UserRepository;
use TranquilCity\TranquilPavement\Infrastructure\Services\Twitter;

class TwitterController extends Controller
{
    public function loginUrl(Request $request, Twitter $twitter)
    {
        try {
            return ['url' => $twitter->getLoginUrl($request)];
        } catch (TwitterOAuthException $e) {
            return Response::create('Twitter exception: ' . $e->getMessage(), 500);
        }
    }

    public function callback(Request $request, Twitter $twitter, UserRepository $repository)
    {
        /** @var User $user */
        $user = $request->user();
        try {
            $twitterUser = $twitter->getOAuthToken($request);
            if (isset($twitterUser['oauth_token'])) {
                $user->setTwitterToken($twitterUser['oauth_token']);
                $user->setTwitterTokenSecret($twitterUser['oauth_token_secret']);
                $repository->save($user);
            }
            return redirect('/admin');
        } catch (\Exception $e) {
            return Response::create('Twitter exception: ' . $e->getMessage(), 500);
        }
    }
}
