<?php

return [
    'TranquilCity\TranquilPavement\Domain\Models\User' => [
        'type'   => 'entity',
        'table'  => 'users',
        'id'     => [
            'uid' => [
                'type'     => 'integer',
                'generator' => [
                    'strategy' => 'auto'
                ]
            ],
        ],
        'uniqueConstraints' => [
            [
                'name' => 'email_unique_idx',
                'columns' => ['email']
            ]
        ],
        'fields' => [
            'email' => [
                'type' => 'string'
            ],
            'passwordHash' => [
                'type' => 'string'
            ],
            'token' => [
                'type' => 'string'
            ],
            'twitterToken' => [
                'type' => 'string',
                'nullable' => true
            ],
            'twitterTokenSecret' => [
                'type' => 'string',
                'nullable' => true
            ],
            'instagramBusinessId' => [
                'type' => 'string',
                'nullable' => true
            ]
        ]
    ],
    'TranquilCity\TranquilPavement\Domain\Models\TranquilPlace' => [
        'type'   => 'entity',
        'table'  => 'places',
        'id'     => [
            'uid' => [
                'type'     => 'integer',
                'generator' => [
                    'strategy' => 'auto'
                ]
            ],
        ],
        'uniqueConstraints' => [
            [
                'name' => 'place_unique_idx',
                'columns' => ['name', 'address']
            ],
            [
                'name' => 'instagram_unique_idx',
                'columns' => ['instagram_id']
            ]
        ],
        'fields' => [
            'name' => [
                'type' => 'string'
            ],
            'slug' => [
                'type' => 'string'
            ],
            'address' => [
                'type' => 'string',
                'nullable' => true
            ],
            'postcode' => [
                'type' => 'string',
                'nullable' => true
            ],
            'geolocation' => [
                'type' => 'point',
                'nullable' => true
            ],
            'imageUrl' => [
                'type' => 'text'
            ],
            'instagramId' => [
                'type' => 'string',
                'nullable' => true
            ],

            'twitterId' => [
                'type' => 'string',
                'nullable' => true
            ],
            'externalUrl' => [
                'type' => 'string'
            ],
            'author' => [
                'type' => 'string',
                'nullable' => true
            ],
            'authorImageUrl' => [
                'type' => 'string',
                'nullable' => true
            ],
            'createdTime' => [
                'type' => 'bigint',
                'nullable' => true
            ],
            'caption' => [
                'type' => 'text',
                'nullable' => true
            ],
            'hidden' => [
                'type' => 'boolean'
            ],
            'events' => [
                'type' => 'boolean',
                'nullable' => false
            ],
            'eventsCaption' => [
                'type' => 'text',
                'nullable' => true
            ],
            'threatened' => [
                'type' => 'boolean',
                'nullable' => true
            ],
            'threatenedCaption' => [
                'type' => 'text',
                'nullable' => true
            ],
            'volunteerNeed' => [
                'type' => 'boolean',
                'nullable' => false
            ],
            'volunteerNeedCaption' => [
                'type' => 'text',
                'nullable' => true
            ],
            'walks' => [
                'type' => 'boolean',
                'nullable' => false
            ],
            'walksCaption' => [
                'type' => 'text',
                'nullable' => true
            ],
            'numComments' => [
                'type' => 'integer',
                'nullable' => true
            ],
            'numLikes' => [
                'type' => 'integer'
            ],
            'numRetweets' => [
                'type' => 'integer',
                'nullable' => true
            ],
            'defaultIndex' => [
                'type' => 'integer',
                'nullable' => true
            ]
        ],
        'manyToOne' => [
            'pollutionData' => [
                'targetEntity' => 'TranquilCity\TranquilPavement\Domain\Models\PollutionDataPoint',
                'joinColumn' => [
                    'name' => 'pollution_data_uid',
                    'referencedColumnName' => 'uid'
                ]
            ]
        ]
    ],
    'TranquilCity\TranquilPavement\Domain\Models\PollutionDataPoint' => [
        'type'   => 'entity',
        'table'  => 'points',
        'id'     => [
            'uid' => [
                'type'     => 'integer',
                'generator' => [
                    'strategy' => 'auto'
                ]
            ],
        ],
        'uniqueConstraints' => [
            [
                'name' => 'point_unique_idx',
                'columns' => ['geolocation']
            ]
        ],
        'fields' => [
            'geolocation' => [
                'type' => 'point'
            ],
            'arri_db' => [
                'type' => 'float'
            ],
            'no2' => [
                'type' => 'float'
            ],
            'pm25' => [
                'type' => 'float'
            ]
        ]
    ]
];
