<?php

namespace Database\Migrations;

use Doctrine\Migrations\AbstractMigration;
use Doctrine\DBAL\Schema\Schema;

class Version20171217141351 extends AbstractMigration
{
    /**
     * @param Schema $schema
     */
    public function up(Schema $schema): void
    {
        $this->abortIf($this->connection->getDatabasePlatform()->getName() != 'postgresql', 'Migration can only be executed safely on \'postgresql\'.');

        // Manually added by Joaquim
        $this->addSql('CREATE EXTENSION IF NOT EXISTS postgis');

        $this->addSql('CREATE SEQUENCE users_uid_seq INCREMENT BY 1 MINVALUE 1 START 1');
        $this->addSql('CREATE SEQUENCE places_uid_seq INCREMENT BY 1 MINVALUE 1 START 1');
        $this->addSql('CREATE SEQUENCE points_uid_seq INCREMENT BY 1 MINVALUE 1 START 1');
        $this->addSql('CREATE TABLE users (uid INT NOT NULL, email VARCHAR(255) NOT NULL, password_hash VARCHAR(255) NOT NULL, token VARCHAR(255) NOT NULL, instagram_token VARCHAR(255) DEFAULT NULL, PRIMARY KEY(uid))');
        $this->addSql('CREATE UNIQUE INDEX email_unique_idx ON users (email)');
        $this->addSql('CREATE TABLE places (uid INT NOT NULL, pollution_data_uid INT DEFAULT NULL, name VARCHAR(255) NOT NULL, slug VARCHAR(255) NOT NULL, address VARCHAR(255) DEFAULT NULL, postcode VARCHAR(255) DEFAULT NULL, geolocation geometry(Point,4326) DEFAULT NULL, image_url VARCHAR(255) NOT NULL, instagram_id VARCHAR(255) NOT NULL, instagram_post_url VARCHAR(255) NOT NULL, author VARCHAR(255) NOT NULL, author_image_url VARCHAR(255) NOT NULL, created_time BIGINT NOT NULL, caption TEXT, PRIMARY KEY(uid))');
        $this->addSql('CREATE INDEX IDX_FEAF6C55F01AF075 ON places (pollution_data_uid)');
        $this->addSql('CREATE UNIQUE INDEX place_unique_idx ON places (name, address)');
        $this->addSql('CREATE UNIQUE INDEX instagram_unique_idx ON places (instagram_id)');
        $this->addSql('CREATE TABLE points (uid INT NOT NULL, geolocation geometry(Point,4326) NOT NULL, arri_db DOUBLE PRECISION NOT NULL, no2 DOUBLE PRECISION NOT NULL, pm25 DOUBLE PRECISION NOT NULL, PRIMARY KEY(uid))');
        $this->addSql('CREATE UNIQUE INDEX point_unique_idx ON points (geolocation)');
        $this->addSql('ALTER TABLE places ADD CONSTRAINT FK_FEAF6C55F01AF075 FOREIGN KEY (pollution_data_uid) REFERENCES points (uid) NOT DEFERRABLE INITIALLY IMMEDIATE');

        // Manually added by Joaquim
        $this->addSql('CREATE INDEX places_gix ON places USING GIST (geolocation)');
        $this->addSql('CREATE INDEX points_gix ON points USING GIST (geolocation)');
    }

    /**
     * @param Schema $schema
     */
    public function down(Schema $schema): void
    {
        $this->abortIf($this->connection->getDatabasePlatform()->getName() != 'postgresql', 'Migration can only be executed safely on \'postgresql\'.');

        $this->addSql('ALTER TABLE places DROP CONSTRAINT FK_FEAF6C55F01AF075');
        $this->addSql('DROP SEQUENCE users_uid_seq CASCADE');
        $this->addSql('DROP SEQUENCE places_uid_seq CASCADE');
        $this->addSql('DROP SEQUENCE points_uid_seq CASCADE');
        $this->addSql('DROP TABLE users');
        $this->addSql('DROP TABLE places');
        $this->addSql('DROP TABLE points');
    }
}
