# Tranquil Pavement

## Requirements

- node 8+
- php 7.1+

## Project Set Up

1. Clone the project
2. `composer install`
3. `npm install`
4. Create a Postgres database
5. `cp .env.example .env`
6. Enter your database credentials to `.env`
7. Add the "tranquil pavement instagram token" from Zoho Vault to `.env`
8. `php artisan doctrine:migrations:migrate`
9. `php artisan db:seed`
10. `php artisan tp:download:pollution`
11. `php artisan tp:import:pollution`
12. `php artisan tp:import:places`
13. `php artisan doctrine:generate:proxies` //this is required to generate proxy classes

## Docker Project Setup

### Setup


Before running the application you will want to have setup the Outlandish NGINX Proxy for Docker.
You can find the project and the instructions for doing so here.

https://nginx-proxy.outlandish.com

Once you have the Outlandish NGINX Proxy for Docker project running locally, you can create SSL certificates
for this project by running

```shell
./bin/create-certificate tranquil.dev
```

Once you have done this and you can also add this domain and some subdomains to your `/etc/hosts`

```shell
echo "127.0.0.1 tranquil.dev db.tranquil.dev mail.tranquil.dev" >> /etc/hosts
```

Once you have created the `tranquil.dev` SSL certificates you can continue with the setup below:

### Configuration

This project relies on a number of third-party services, and so a number of configuration is required.

1. Clone the project
2. Copy the `.env.example` to `.env` with the following command `cp .env.example .env`
3. `GOOGLE_MAPS_API_KEY`: Create a new Project for Tranquil Pavement at https://console.cloud.google.com/projectcreate 
and enable the Geocoding API and copy the key into place
4. `TWITTER_CONSUMER_KEY`: This is provided for you in the `.env.example` and will work with the url https://tranquil.dev. 
5. `TWITTER_CONSUMER_SECRET`: This is provided for you in the `.env.example` and will work with the url https://tranquil.dev.
6. `FACEBOOK_APP_ID`: Copy this from the `.env` file on the production instance
7. `FACEBOOK_APP_SECRET`: Copy this from the `.env` file on the production instance
8. `INSTAGRAM_BUSINESS_ID`: Copy this from the `.env` file on the production instance
9. `INSTAGRAM_API_TOKEN`: Copy this from the `.env` file on the production instance

### Install Dependencies

1. Run `docker-compose up tools` to run `composer install` and migrate the database.
2. Run `docker-compose run --rm node npm i`

### Running the project

1. Run the project with `docker-compose up -d web node`. This will start the web service that runs 
NGINX and PHP as well as the Mail server and Database, and then also the Assets service that will generate the
frontend assets and watch for any changes.

> If this is your first time running this command, it will take some
> time to install the node dependencies and build the assets on the 
> frontend. Use `docker-compose logs -f node` to track the logging
> from the node service, to review its progress.

### Seeding the database

After you have the project running, you will need to seed the database with some data. 

1. Run `docker-compose run --rm tools php artisan db:seed` to create some users
2. Run `docker-compose run --rm tools php artisan tp:download:pollution` to download 
the pollution dataset from S3
3. Run `docker-compose run --rm tools php artisan tp:import:pollution` to import the 
pollution dataset into the database. This will take a long time

Before you can continue you will now need to login to a user account on your local instance 
at https://tranquil.dev/login (see [`UserSeeder`](./database/seeds/UserSeeder.php) for credentials)
and then login to Twitter to add a Twitter API token to the database for that user. This will be
used to fetch Places from Twitter in the next step. 

1. Run `docker-compose run --rm tools php artisan tp:import:places` to fetch places from 
Twitter and Instagram. If no places are fetched from either source, you may need to add a
post that is tagged with `tranquilcitylondon` to either Twitter or Instagram. 
   
## tp:import:places command

This command uses the APIs for Twitter and Instagram/Facebook to fetch recent posts from both
services that have been tagged with `tranquilcitylondon` and loads them into the database to 
be shown on the map. 

### Twitter authentication

The authentication for twitter uses an access token stored in the database against one of the 
users. This requires one of the admin users of the platform to login to Twitter using the button
found on /admin.

### Instagram authentication

To scrape Instagram for photos, the app needs an Instagram API Token. This
should be stored in the `.env` file.

Follow the instructions here: https://medium.com/@yasithlokuge/how-to-generate-a-never-expiring-facebook-page-access-token-24ac5c1a95f1.

The production site uses a previously generated access token that has no expiry. If this stops working,
whoever is going to get a new access token needs to first be made an admin of the Tranquil Pavement Facebook App
and the Tranquil City Facebook Page. Then it should be possible to follow the above steps to get a new token.

## Adding database migrations

Because the existing Doctrine migrations (class names, file names) don't follow Laravel convention,
you should not use the Laravel-native commands or helpers to build migrations. Instead mimic the
format in `database/migrations` or use Doctrine's native commands to make the migration scaffolding,
and run tests with the `doctrine:` namespace commands instead of `laravel migrate`.
