const angular = require('angular')
angular
  .module('TranquilPavement')
  .directive('navbar', ['$cookies', 'LayersService', function ($cookies, LayersService) {
    return {
      restrict: 'E',
      scope: {
        onSearch: '=',
        onClickLogo: '='
      },
      template: `
        <div class="cookiebar" ng-if="showCookieBar">
            <span class="cookiebar__text">This site uses cookies: <a href="/privacy.html" target="_blank">Find out more</a></span>
            <button class="cookiebar__close" ng-click="hideCookieBar()">&times</button>
        </div>
        <div class="navbar">
            <div class="navbar__brand" ng-click="onClickLogo()"><strong>Tranquil Pavement</strong> London</div>
            <form class="navbar__search" ng-submit="doSearch({ q: q })">
                <span class="fa fa-location-arrow"></span>
                <input id="navbar-search" type="text" ng-model="q" title="Search by location">
                <button><span class="fa fa-search"></span></button>
            </form>

            <i class="fa fa-bars" ng-click="layerMenuOpen = !layerMenuOpen"></i>
            <div ng-if="layerMenuOpen">
                <div class="navbar__layer-menu">
                  <h3>Select Layer</h3>
                  <ul>
                      <li ng-repeat="tileset in getTilesets() track by $index"><a ng-click="setSelectedTileset(tileset)" ng-class="{ 'active': isActive(tileset) }">{{ tileset.name }}</a></li>
                  </ul>
                </div>
            </div>
        </div>`,
      link: function (scope) {
        scope.layerMenuOpen = false
        scope.showCookieBar = !$cookies.get('acceptCookies')
        scope.hideCookieBar = function () {
          scope.showCookieBar = false
          document.getElementsByClassName('map__container')[0].style.height = 'auto'
          $cookies.put('acceptCookies', true)
        }
        scope.doSearch = function (q) {
          scope.onSearch(q)
          // hide keyboard on mobile
          const input = document.getElementById('navbar-search')
          input.focus()
          input.blur()
        }
        scope.isActive = function (tileset) {
          return LayersService.getSelectedTileset().name === tileset.name
        }
        scope.getTilesets = LayersService.getTilesets
        scope.setSelectedTileset = function (tileset) {
          LayersService.setSelectedTileset(tileset)
          scope.layerMenuOpen = false
        }
      }
    }
  }])
