const angular = require('angular')
angular
  .module('TranquilPavement')
  .directive('placeCard', ['$timeout', '$state', '$stateParams', '$transitions', 'leafletData',
    function ($timeout, $state, $stateParams, $transitions, leafletData) {
      return {
        restrict: 'E',
        scope: {
          place: '=',
          onClose: '=',
          nav: '='
        },
        template: `
        <card view="{{ state !== 'instagram' ? 'place' : '' }}" hide-card="hideCard" ng-if="state !== 'data'" ng-swipe-left="prev()" ng-swipe-right="next()">
          <div ng-if="state === 'instagram'" class="card__instagram card__instagram--place">
            <h1>View post on {{ params.service }}</h1> 
            <p ng-if="!params.tag && !params.user">You are about to leave Tranquil Pavement to view the original post on {{ params.service }}.</p>
            <p ng-if="params.tag">You about to visit {{ params.service }} to see other posts that share the '{{ params.tag }}' tag.</p>
            <p ng-if="params.user">You about to visit {{ params.service }} to view {{ params.user}}'s profile.</p>
            <button class="button button--secondary" type="button" ng-click="updateView('image')">Back</button>
            <a class="button button--primary" ng-href="{{ params.destination }}">Continue</a>
          </div>
          <div ng-if="state !== 'instagram'">
            <div class="place-info">
              <div class="place-info__header">
                <div class="place-info__author">
                    <img ng-if="place.authorImageUrl" ng-src="{{ place.authorImageUrl }}" alt="{{ place.author }}">
                    <p ng-if="place.author" class="place-info__author-name" ng-click="goToInstagramOrTwitter('user', place.author)">@{{ place.author }}</p>
                </div>
                <div class="place-info__meta">
                    <p ng-if="place.createdTime" class="place-info__meta-date">{{ place.createdTime * 1000 | date : 'd MMMM yyyy' }}</p>
                    <p ng-if="place.distance" class="place-info__meta-distance">{{ (place.distance / 1000).toFixed(2) }}km</p>
                </div>
              </div>
              <div class="place-info__image">
                 <a class="place-info__prev" ui-sref="app.image({ slug: nav.prev })">&lt;</a>
                 <img ng-src="{{ place.imageUrl }}" ng-click="goToInstagramOrTwitter()" alt="{{ place.name }}" image-load="updateMapHeight()">
                 <span ng-if="place.threatened" class="place-info__threatened-bar">THREATENED SPACE</span>
                 <a class="place-info__next" ui-sref="app.image({ slug: nav.next })">&gt;</a>
              </div>
              <div class="place-info__name">
                  <p><span class="fa fa-map-marker"></span>{{ place.name }}</p>
              </div>
              <p ng-if="place.captionParts" class="place-info__caption">
                  <span ng-repeat="part in place.captionParts">
                      <a ng-if="part.type === 'tag'" ng-click="goToInstagramOrTwitter('tag', part.link)" href="#">{{ part.text }}</a>
                      <a ng-if="part.type === 'user'" ng-click="goToInstagramOrTwitter('user', part.link)" href="#">{{ part.text }}</a>
                      <span ng-if="part.type == 'normal'">{{ part.text }}</span>
                  </span>
              </p>
              <div ng-if="place.events">
                  <h5 class="place-info__events-header">Events</h5>
                  <p class="place-info__events-caption">
                      <span ng-repeat="part in place.eventsCaptionParts">
                          <a ng-if="part.type === 'link'" ng-href="{{ part.text }}" target="_blank">{{ part.text }}</a>
                          <a ng-if="part.type === 'email'" ng-href="mailto:{{ part.text }}">{{ part.text }}</a>
                          <span ng-if="part.type === 'normal'">{{ part.text }}</span>
                          <br ng-if="part.type === 'linebreak'">
                      </span>
                  </p>
              </div>
              <div ng-if="place.threatened">
                  <h5 class="place-info__threatened-header">Threatened Space</h5>
                  <p class="place-info__threatened-caption">
                      <span ng-repeat="part in place.threatenedCaptionParts">
                          <a ng-if="part.type === 'link'" ng-href="{{ part.text }}" target="_blank">{{ part.text }}</a>
                          <a ng-if="part.type === 'email'" ng-href="mailto:{{ part.text }}">{{ part.text }}</a>
                          <span ng-if="part.type === 'normal'">{{ part.text }}</span>
                          <br ng-if="part.type === 'linebreak'">
                      </span>
                  </p>
              </div>
              <div ng-if="place.volunteerNeed">
                  <h5 class="place-info__volunteer-need-header">Community Hub</h5>
                  <p class="place-info__volunteer-need-caption">
                      <span ng-repeat="part in place.volunteerNeedCaptionParts">
                          <a ng-if="part.type === 'link'" ng-href="{{ part.text }}" target="_blank">{{ part.text }}</a>
                          <a ng-if="part.type === 'email'" ng-href="mailto:{{ part.text }}">{{ part.text }}</a>
                          <span ng-if="part.type === 'normal'">{{ part.text }}</span>
                          <br ng-if="part.type === 'linebreak'">
                      </span>
                  </p>
              </div>
              <div ng-if="place.walks">
                  <h5 class="place-info__walks-header">Walks</h5>
                  <p class="place-info__walks-caption">
                      <span ng-repeat="part in place.walksCaptionParts">
                          <a ng-if="part.type === 'link'" ng-href="{{ part.text }}" target="_blank">{{ part.text }}</a>
                          <a ng-if="part.type === 'email'" ng-href="mailto:{{ part.text }}">{{ part.text }}</a>
                          <span ng-if="part.type === 'normal'">{{ part.text }}</span>
                          <br ng-if="part.type === 'linebreak'">
                      </span>
                  </p>
              </div>
              <div class="place-info__details">
                  <button type="button" ng-click="goToState('app.pollution')">View Pollution Data <span class="fa fa-arrow-right"></span></button>
              </div>
            </div>
          </div>
        </card>
        <div class="pollution-info" ng-if="state === 'data'">
          <div class="pollution-info__content">
            <h2>Pollution Data</h2>
            <div class="pollution-info__location" ng-if="place.address || place.location">
              <h3>Location</h3>
              <p>{{ (place.address ? place.address + ' ' : '') + (place.postcode || '') }}</p>
            </div>
            <div class="pollution-info__box">
              <span class="fa fa-bullhorn"></span>
              <h3>Noise pollution<small><sup>&nbsp;[1]</sup></small></h3>
              <p><span class="pollution-info__number">{{ (place.pollutionData.arri_db < 50 ? '< 50' : place.pollutionData.arri_db.toFixed(1)) }}</span>dBA</p>
              <p class="pollution-info__guidelines">
              {{ place.pollutionData.arri_db < 50 ? 'Low ' : (place.pollutionData.arri_db < 60 ? 'Moderate ' : 'Affected by ') }}noise
              </p>
            </div>
            <div class="pollution-info__box">
              <span class="fa fa-industry"></span>
              <h3>Air pollution<small><sup>&nbsp;[2]</sup></small></h3>
              <div class="pollution-info__row">
                <div>
                  <h4>PM<sub>2.5</sub></h4>
                  <p><span class="pollution-info__number">{{ place.pollutionData.pm25 | number : '0'}}</span>&micro;g/m<sup>3</sup></p>
                  <p class="pollution-info__guidelines">{{ place.pollutionData.pm25 > 20 ? 'Above ' : 'Below '}}WHO guideline of 20&micro;g/m<sup>3</sup></p>
                </div>
                <div>
                  <h4>NO<sub>2</sub></h4>
                  <p><span class="pollution-info__number">{{ place.pollutionData.no2 | number : '0'}}</span>&micro;g/m<sup>3</sup></p>
                  <p class="pollution-info__guidelines">{{ place.pollutionData.no2 > 40 ? 'Above ' : 'Below '}}WHO guideline of 40&micro;g/m<sup>3</sup></p>
                </div>
              </div>
            </div>
            <div class="pollution-info__sources">
              <p>[1] Noise data is taken from the Government Department for Environment, Food and Rural Affairs (Defra).
               Noise Mapping England project and shows the combined modelled annual average noise levels from road, rail, 
               aircraft and industrial sources. Unfortunately, no natural or positive sounds are included.
              </p>
              <p>[2] Air pollution data is taken from the Greater London Authority (GLA) London Atmospheric Emissions 
              Inventory (LAEI) and shows modelled annual average concentrations of nitrogen dioxide (NO<sub>2</sub>) and 
              particulate matter less than 2.5 microns in aerodynamic diameter (PM<sub>2.5</sub>). PM<sub>2.5</sub> is the
              pollutant most strongly associated with direct health effects.
              </p>
            </div>
            <div class="pollution-info__back">
                <button type="button" ng-click="goToState('app.image')"><span class="fa fa-arrow-left"></span>Back</button>
            </div>
          </div>
        </div>`,
        link: function ($scope) {
          $scope.$watch('place', () => {
            $scope.place.captionParts = processCaption($scope.place.caption)
            $scope.place.eventsCaptionParts = processAuxiliaryCaption($scope.place.eventsCaption)
            $scope.place.threatenedCaptionParts = processAuxiliaryCaption($scope.place.threatenedCaption)
            $scope.place.volunteerNeedCaptionParts = processAuxiliaryCaption($scope.place.volunteerNeedCaption)
            $scope.place.walksCaptionParts = processAuxiliaryCaption($scope.place.walksCaption)
            $scope.updateMapHeight()
          })

          $scope.state = $state.current.name === 'app.pollution' ? 'data' : 'image'
          $scope.params = {}

          $scope.hideCard = function () {
            $scope.state = 'hidden'
            $scope.onClose()
          }

          $scope.goToState = function (state) {
            $state.go(state, { slug: $scope.place.slug }, { notify: true })
          }

          $scope.updateView = function (state, params = {}) {
            $scope.state = state
            $scope.params = params
          }

          $scope.next = function () {
            $state.go('app.image', { slug: $scope.nav.next })
          }

          $scope.prev = function () {
            $state.go('app.image', { slug: $scope.nav.prev })
          }

          angular.element(document).on('keyup', function (event) {
            if (event.keyCode === 37) {
              $scope.prev()
            }
            if (event.keyCode === 39) {
              $scope.next()
            }
          })

          $scope.$on('$destroy', function () {
            angular.element(document).unbind('keyup')
          })

          $scope.$watch('state', function () {
            $scope.updateMapHeight()
          })

          $scope.updateMapHeight = function () {
            const $map = angular.element(document.querySelector('.map__container'))
            if ($scope.state === 'image') {
              $timeout(() => {
                const cardHeight = angular.element(document.querySelector('.card'))[0].offsetHeight
                $map.css('height', cardHeight + 'px')
                updateMap(cardHeight)
              }, 100)
            } else {
              window.scrollTo(0, 0)
              $map.css('height', 'auto')
              updateMap()
            }
          }

          $scope.goToInstagramOrTwitter = function (type, text) {
            const isTwitter = Boolean($scope.place.twitterId)
            const params = {
              service: isTwitter ? 'Twitter' : 'Instagram',
              destination: $scope.place.externalUrl
            }
            if (type === 'tag') {
              params.tag = text
              params.destination = isTwitter ? 'https://twitter.com/search?q=%23' + text
                : 'https://www.instagram.com/explore/tags/' + text + '/'
            }
            if (type === 'user') {
              params.user = text
              params.destination = isTwitter ? 'https://twitter.com/' + text
                : 'https://www.instagram.com/_u/' + text + '/'
            }
            $scope.updateView('instagram', params)
          }

          function updateMap (cardHeight) {
            leafletData
            .getMap()
            .then(function (map) {
              map.invalidateSize()
              if ($scope.state === 'image') {
                // get the pixel coordinates of the point, add half the card height to it, convert back to lat/lng
                // this centers with the point at the top of the card
                const point = map.latLngToLayerPoint({
                  lat: $scope.place.geolocation.latitude,
                  lng: $scope.place.geolocation.longitude
                })
                const coords = map.layerPointToLatLng({ x: point.x, y: (point.y + cardHeight / 2 - 50) })
                map.setView(coords)
              }
            })
          }

          function processCaption (caption) {
            if (caption) {
              caption = caption.replace(/\s/g, ' ')
              caption = caption.replace(/&amp;/g, '&')

              const tags = (caption.match(/#\w+/g) || []).map(tag => tag.substring(1).trim())
              const users = (caption.match(/@\w+/g) || []).map(user => user.substring(1).trim())
              const words = (caption.split(/ /g) || []).map(word => word.trim()).filter(Boolean)

              return words.map(word => {
                const cleanWord = word.replace(/\W/g, '')
                const tagOrUser = word.indexOf('#') > -1 || word.indexOf('@') > -1
                const isTag = tagOrUser && tags.indexOf(cleanWord) > -1
                const isUser = tagOrUser && users.indexOf(cleanWord) > -1
                const type = isTag ? 'tag' : (isUser ? 'user' : 'normal')
                return {
                  type,
                  text: word,
                  link: cleanWord
                }
              })
            } else {
              return []
            }
          }

          function processAuxiliaryCaption (caption) {
            if (caption) {
              caption = caption.replace(/(\r\n|\r|\n)/g, ' <br> ')
              const words = (caption.split(/ /g) || []).map(word => word.trim()).filter(Boolean)
              return words.map(text => {
                const isLink = text.indexOf('http') === 0 && text.indexOf('.') > -1
                const isEmail = text.indexOf('@') > 0 && text.length > 2
                const isLineBreak = text === '<br>'
                const type = isLink ? 'link' : (isLineBreak ? 'linebreak' : (isEmail ? 'email' : 'normal'))
                return { type, text }
              })
            } else {
              return []
            }
          }

          $transitions.onSuccess({}, () => {
            if ($state.current.name === 'app.pollution') {
              $scope.updateView('data')
            } else {
              $scope.updateView('image')
            }
          })
        }
      }
    }])
