const angular = require('angular')

angular.module('TranquilPavement').service('TranquilPlacesService', ['$http', function ($http) {
  this.get = function (uid) {
    return $http
      .get('/api/places/' + uid)
      .then(res => res.data)
  }

  this.list = function (timeOrder) {
    return $http
      .get('/api/places?order=' + (timeOrder ? 'createdTime' : 'defaultIndex'))
      .then(res => res.data)
  }

  this.search = function (query) {
    const q = query.q ? prepareSearchQuery(query.q) : ''
    const latitude = query.latitude ? encodeURIComponent(query.latitude) : ''
    const longitude = query.longitude ? encodeURIComponent(query.longitude) : ''

    const url = '/api/places/search?q=' + q + '&latitude=' + latitude + '&longitude=' + longitude

    return $http
      .get(url)
      .then(res => res.data)
  }

  this.create = function (url) {
    return $http
      .post('/api/places', { url })
      .then(res => res.data)
  }

  this.update = function (uid, place) {
    return $http
      .put('/api/places/' + uid, place)
      .then(res => res.data)
  }

  function prepareSearchQuery (q) {
    return encodeURIComponent(q)
  }
}])
