const angular = require('angular')

angular
  .module('TranquilPavement')
  .component('adminEdit', {
    template: `
    <div class="admin">
      <div class="admin__header">
        <h1>Tranquil Pavement Admin</h1>
        <a ui-sref="adminList">Back</a>
      </div>
      <h2>Edit Tranquil Place</h2>
      <p><a href="{{ place.externalUrl }}" target="_blank">{{ place.externalUrl }}</a></p>
      <div class="admin-edit__image-bar">
        <div class="admin-edit__image" style="background-image:url({{ place.imageUrl }})">
        </div>
        <leaflet defaults="mapConfig" lf-center="mapCenter" markers="markers" class="admin-edit__map"></leaflet>
      </div>
      <form ng-submit="update()">
        <div class="admin-edit__form-item">
          <label for="name">Place Name</label>
          <input type="text" id="name" ng-model="place.name">
        </div>
        <div class="admin-edit__form-item">
          <label for="address">Address</label>
          <input type="text" id="address" ng-model="place.address">
        </div>
        <div class="admin-edit__form-item">
          <label for="postcode">Postcode</label>
          <input type="text" id="postcode" ng-model="place.postcode">
        </div>
        <div class="admin-edit__form-item">
          <label for="lat">Latitude</label>
          <input id="lat" type="number" step="any" ng-model="markers.place.lat">
        </div>
        <div class="admin-edit__form-item">
          <label for="lng">Longitude</label>
          <input id="lng" type="number" step="any" ng-model="markers.place.lng">
        </div>
        <div class="admin-edit__form-item">
          <label for="caption">Caption</label>
          <textarea id="caption" ng-model="place.caption"></textarea>
        </div>

        <div class="admin-edit__form-item">
          <label for="events">Events?</label>
          <input id="events" type="checkbox" ng-model="place.events">
        </div>
        <div class="admin-edit__form-item" ng-if="place.events">
          <label for="events-caption">Event Details</label>
          <textarea id="events-caption" ng-model="place.eventsCaption"></textarea>
        </div>

        <div class="admin-edit__form-item">
          <label for="threatened">Threatened?</label>
          <input id="threatened" type="checkbox" ng-model="place.threatened">
        </div>
        <div class="admin-edit__form-item" ng-if="place.threatened">
          <label for="threatened-caption">Threatened Caption</label>
          <textarea id="threatened-caption" ng-model="place.threatenedCaption"></textarea>
        </div>

        <div class="admin-edit__form-item">
          <label for="volunteer-need">Volunteers Needed?</label>
          <input id="volunteer-need" type="checkbox" ng-model="place.volunteerNeed">
        </div>
        <div class="admin-edit__form-item" ng-if="place.volunteerNeed">
          <label for="volunteer-need-caption">Volunteer Need Details</label>
          <textarea id="volunteer-need-caption" ng-model="place.volunteerNeedCaption"></textarea>
        </div>

        <div class="admin-edit__form-item">
          <label for="walks">Walks?</label>
          <input id="walks" type="checkbox" ng-model="place.walks">
        </div>
        <div class="admin-edit__form-item" ng-if="place.walks">
          <label for="walks-caption">Walk Details</label>
          <textarea id="walks-caption" ng-model="place.walksCaption"></textarea>
        </div>

        <div class="admin-edit__form-item">
          <label for="hidden">Hide</label>
          <input id="hidden" type="checkbox" ng-model="place.hidden">
        </div>
        <div class="admin-edit__form-item">
          <button type="submit" ng-disabled="loading">Update</button>
          <button type="button" ng-disabled="loading" ng-click="cancel()">Cancel</button>
          <span>{{ error }}</span>
        </div>
      </form>
    </div>
    `,
    controller: ['$scope', '$state', '$stateParams', 'TranquilPlacesService',
      function ($scope, $state, $stateParams, TranquilPlacesService) {
        angular.extend($scope, {
          loading: false,
          mapConfig: {
            tileLayer: 'https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png',
            minZoom: 10,
            maxZoom: 16
          },
          mapCenter: {
            lat: 51.51,
            lng: -0.12,
            zoom: 13
          },
          markers: {},
          update: function () {
            $scope.loading = true

            $scope.place.geolocation.latitude = $scope.markers.place.lat
            $scope.place.geolocation.longitude = $scope.markers.place.lng

            const place = angular.copy($scope.place)
            place.tags = place.tags.split(' ')

            TranquilPlacesService
              .update($stateParams.uid, place)
              .then(() => {
                $state.go('adminList')
              }, res => {
                if (res && res.data) {
                  $scope.error = res.data.message || res.data
                } else {
                  $scope.error = res.statusText
                }
              })
          },
          cancel: function () {
            setPlace($scope.originalPlace)
          }
        })

        TranquilPlacesService
          .get($stateParams.uid)
          .then(place => {
            setPlace(place)
          })

        function setPlace (place) {
          $scope.originalPlace = angular.copy(place)

          setupPlace(place)

          $scope.place = place
          $scope.mapCenter.lat = place.geolocation.latitude
          $scope.mapCenter.lng = place.geolocation.longitude

          $scope.markers.place = {
            lat: place.geolocation.latitude,
            lng: place.geolocation.longitude,
            message: place.name || 'Unknown place',
            focus: false,
            draggable: true,
            icon: {
              iconUrl: '/images/icons/map-pin.svg',
              iconSize: [40, 40],
              iconAnchor: [20, 40]
            }
          }
        }

        // Add the default location to the place if it doesn't have a location
        function setupPlace (place) {
          if (!place.geolocation) {
            place.geolocation = {}
          }
          if (!place.geolocation.latitude) {
            place.geolocation.latitude = $scope.mapCenter.lat
          }
          if (!place.geolocation.longitude) {
            place.geolocation.longitude = $scope.mapCenter.lng
          }
          place.tags = place.tags ? place.tags.join(' ') : ''
        }

        $scope.$on('leafletDirectiveMarker.dragend', function (e, args) {
          $scope.markers.place.lng = args.model.lng
          $scope.markers.place.lat = args.model.lat
        })
      }]
  })
