const angular = require('angular')

angular
  .module('TranquilPavement')
  .component('adminList', {
    template: `
    <div class="admin">
      <div class="admin__header">
        <h1>Tranquil Pavement Admin</h1>
        <span ng-if="!user">Loading...</span>
        <div class="admin__links" ng-if="user">
          <a href="/api/places/download" target="_blank" download>Download CSV</a>
          <a ng-if="user.twitterToken" ui-sref="adminCreate">Add Place</a>
          <a ng-if="!user.twitterToken" href="#" ng-click="logInWithTwitter()">Log in with Twitter</a>
          <div class="fb-login-button" data-width="100" data-max-rows="1" data-size="small"
               data-button-type="login_with" data-show-faces="false" data-auto-logout-link="true"
               data-use-continue-as="false" onlogin="getLoginStatus()"
               data-scope="public_profile,manage_pages,pages_show_list,instagram_basic,instagram_manage_comments,instagram_manage_insights"></div>
          <a ng-if="user" href="#" ng-click="logout()">Log Out</a>
        </div>
      </div>
      <p ng-if="loadingPlaces">Loading...</p>
      <div class="admin-list__filter">
        <label for="searchText">Search:&nbsp;</label>
        <input type="text" ng-model="search.details" name="searchText" id="searchText">
      </div>
      <table ng-if="user && places">
        <thead>
          <tr>
            <th class="admin-list__name">Name</th>
            <th class="admin-list__image">Image</th>
          </tr>
        </thead>
        <tbody>
          <tr class="admin-list__row" ng-repeat="place in places | filter:search:strict" ng-click="edit(place.uid)">
            <td>{{ place.details }}</td>
            <td class="admin-list__image"><img ng-src="{{ place.imageUrl }}" alt="{{ place.name }}"></td>
          </tr>
        </tbody>
      </table>
    </div>
    `,
    controller: ['$scope', '$timeout', '$state', '$localStorage', 'TranquilPlacesService', 'TwitterService', 'UsersService',
      'InstagramService',
      function ($scope, $timeout, $state, $localStorage, TranquilPlacesService, TwitterService, UsersService) {
        $scope.loadingPlaces = false

        UsersService
          .getCurrentUser()
          .then(user => {
            $scope.user = user
            return initFB()
          })
          .then(() => {
            window.getLoginStatus()
          })

        TranquilPlacesService
          .list(true)
          .then(places => {
            $scope.places = places.map(p => {
              p.details = getPlaceDetails(p)
              return p
            })
          })

        $scope.logInWithTwitter = function () {
          TwitterService
            .getLoginUrl()
            .then(url => { window.location = url })
        }

        $scope.edit = function (uid) {
          $state.go('adminEdit', { uid })
        }

        $scope.$watch('search.details', (newVal) => {
          $localStorage.search = newVal
        })

        $scope.search = {
          details: $localStorage.search
        }

        $scope.logout = function () {
          UsersService.logout().then(() => $state.go('app'))
        }

        window.getLoginStatus = function () {
          window.FB.getLoginStatus(response => {
            $timeout(() => {
              $scope.facebookLoaded = true
              $scope.user.facebookToken = response.authResponse && response.authResponse.accessToken
              console.log('user', $scope.user)
            }, 1)
          })
        }

        function initFB () {
          return new Promise((resolve, reject) => {
            initFBRecursive(resolve)
          }).then(() => window.tranquilFbInit())
        }

        function initFBRecursive (callback) {
          if (window.FB) {
            return callback()
          }
          setTimeout(() => initFBRecursive(callback), 100)
        }

        function getPlaceDetails (place) {
          let details = place.name + ' ('
          const hasLocation = place.geolocation && place.geolocation.latitude && place.geolocation.longitude
          if (!place.hidden && hasLocation) {
            details += 'shown)'
            return details
          }
          details += 'hidden, ' + (hasLocation ? 'located' : 'not located') + ')'
          return details
        }
      }]
  })
