<?php

namespace TranquilCity\TranquilPavement\Domain\Models;

/**
 * Class Point
 *
 * Represents a location in the world.
 *
 * @category Class
 * @package  TheRestartProject\RepairDirectory\Domain\Models
 * @author   Joaquim d'Souza <joaquim@outlandish.com>
 * @license  http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * @link     http://www.outlandish.com/
 */
class Point
{
    private $latitude;
    private $longitude;

    /**
     * Point constructor
     *
     * @param float $latitude  The latitude of the point
     * @param float $longitude The longitude of the point
     */
    public function __construct($latitude, $longitude)
    {
        $this->latitude = $latitude;
        $this->longitude = $longitude;
    }

    /**
     * Creates a Point object from an array
     *
     * @param array $geolocation Latitude and Longitude in array
     *
     * @return null|Point
     */
    public static function fromArray($geolocation)
    {
        if (empty($geolocation)) {
            return null;
        }

        if (isset($geolocation['latitude'], $geolocation['longitude'])) {
            return new self($geolocation['latitude'], $geolocation['longitude']);
        }

        return new self($geolocation[0], $geolocation[1]);
    }

    /**
     *
     * Creates a Point object from the average of all the latitudes and longitudes of
     * the points in the provided Multi-Polygon.
     *
     * @param $multiPolygon
     *
     * @return null|Point
     */
    public static function fromBoundingBox($multiPolygon)
    {
        $count = 0;
        $latitudeSum = 0;
        $longitudeSum = 0;
        foreach ($multiPolygon as $polygon) {
            foreach ($polygon as $point) {
                $longitudeSum += $point[0];
                $latitudeSum += $point[1];
                $count++;
            }
        }
        if ($count) {
            return new Point($latitudeSum / $count, $longitudeSum / $count);
        }
        return null;
    }

    /**
     * Return the latitude of the point
     *
     * @return float
     */
    public function getLatitude()
    {
        return $this->latitude;
    }

    /**
     * Return the longitude of the point
     *
     * @return float
     */
    public function getLongitude()
    {
        return $this->longitude;
    }

    /**
     * Convert the instance to a [ key => value ] array.
     *
     * @return array
     */
    public function toArray()
    {
        $array = get_object_vars($this);
        return $array;
    }
}
