<?php

namespace TranquilCity\TranquilPavement\Domain\Models;

class TranquilPlace
{
    private $uid;
    private $name;
    private $slug;
    private $address;
    private $postcode;
    private $geolocation;
    private $instagramId;
    private $twitterId;
    private $externalUrl;
    private $imageUrl;
    private $author;
    private $authorImageUrl;
    private $createdTime;
    private $caption;

    /** @var bool Whether place has event-related "location and info box with links" */
    private $events = false;
    private $eventsCaption;

    private $threatened = false;
    private $threatenedCaption;

    /** @var bool Whether place has info on "help needed to maintain and/or improve a space" */
    private $volunteerNeed = false;
    private $volunteerNeedCaption;

    /** @var bool Whether place has "routes and info box with links to download" */
    private $walks = false;
    private $walksCaption;

    private $pollutionData;
    private $hidden = false;
    private $numLikes = 0;
    private $numComments = null;
    private $numRetweets = null;
    private $defaultIndex;

    /**
     * @return mixed
     */
    public function getUid()
    {
        return $this->uid;
    }

    /**
     * @param mixed $uid
     */
    public function setUid($uid)
    {
        $this->uid = $uid;
    }

    /**
     * @return mixed
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param mixed $name
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * @return mixed
     */
    public function getAddress()
    {
        return $this->address;
    }

    /**
     * @param mixed $address
     */
    public function setAddress($address)
    {
        $this->address = $address;
    }

    /**
     * @return mixed
     */
    public function getPostcode()
    {
        return $this->postcode;
    }

    /**
     * @param mixed $postcode
     */
    public function setPostcode($postcode)
    {
        $this->postcode = $postcode;
    }

    /**
     * @return Point
     */
    public function getGeolocation()
    {
        return $this->geolocation;
    }

    public function hasGeolocation()
    {
        $g = $this->getGeolocation();
        return $g && $g->getLatitude() && $g->getLongitude();
    }

    /**
     * @param mixed $geolocation
     */
    public function setGeolocation($geolocation)
    {
        $this->geolocation = $geolocation;
    }

    /**
     * Convert the instance to a [ key => value ] array.
     *
     * @return array
     */
    public function toArray()
    {
        $array = get_object_vars($this);
        $array['geolocation'] = $this->hasGeolocation() ? $this->getGeolocation()->toArray() : null;

        $pollutionData = $this->getPollutionData();
        if ($pollutionData) {
            $array['pollutionData'] = [
                'pm25' => $this->getPollutionData()->getPm25(),
                'no2' => $this->getPollutionData()->getNo2(),
                'arri_db' => $this->getPollutionData()->getArriDb()
            ];
        }
        return $array;
    }

    public function toCSVRow()
    {
        $array = $this->toArray();
        $array['hidden'] = $array['hidden'] ? 'Y' : 'N';
        $array['events'] = $array['events'] ? 'Y' : 'N';
        $array['threatened'] = $array['threatened'] ? 'Y' : 'N';
        $array['volunteerNeed'] = $array['volunteerNeed'] ? 'Y' : 'N';
        $array['walks'] = $array['walks'] ? 'Y' : 'N';
        $array['caption'] = str_replace(["\n", "\r\n", "\r"], ' ', $array['caption']);
        $array['latitude'] = $array['geolocation']['latitude'];
        $array['longitude'] = $array['geolocation']['longitude'];
        if (array_key_exists('pollutionData', $array)) {
            $array['pm25'] = $array['pollutionData']['pm25'];
            $array['no2'] = $array['pollutionData']['no2'];
            $array['arri_db'] = $array['pollutionData']['arri_db'];
        } else {
            $array['pm25'] = null;
            $array['no2'] = null;
            $array['arri_db'] = null;
        }
        unset($array['geolocation']);
        unset($array['pollutionData']);
        return $array;
    }

    /**
     * @return mixed
     */
    public function getImageUrl()
    {
        return $this->imageUrl;
    }

    /**
     * @param mixed $imageUrl
     */
    public function setImageUrl($imageUrl)
    {
        $this->imageUrl = $imageUrl;
    }

    /**
     * @return mixed
     */
    public function getInstagramId()
    {
        return $this->instagramId;
    }

    /**
     * @param mixed $instagramId
     */
    public function setInstagramId($instagramId)
    {
        $this->instagramId = $instagramId;
    }

    /**
     * @return mixed
     */
    public function getAuthor()
    {
        return $this->author;
    }

    /**
     * @param mixed $author
     */
    public function setAuthor($author)
    {
        $this->author = $author;
    }

    /**
     * @return mixed
     */
    public function getCreatedTime()
    {
        return $this->createdTime;
    }

    /**
     * @param mixed $createdTime
     */
    public function setCreatedTime($createdTime)
    {
        $this->createdTime = $createdTime;
    }

    /**
     * @return mixed
     */
    public function getCaption()
    {
        return $this->caption;
    }

    public function getCaptionLines()
    {
        if (!$this->caption) {
            return [];
        }
        $lines = preg_split('#,|\.|!|-|\r?\n#', $this->caption);
        $filtered = array_values(
            array_filter($lines, function ($line) {
                return trim($line);
            })
        );
        return $filtered;
    }

    public function getCaptionName()
    {
        $lines = $this->getCaptionLines();
        if (count($lines) && $lines[0]) {
            $name = $lines[0];
            if (strpos($name, '?') !== false) {
                $name = explode('?', $name)[0] . '?';
            }
            return substr($name, 0, 255);
        }
        return null;
    }

    /**
     * @param mixed $caption
     */
    public function setCaption($caption)
    {
        $this->caption = $caption;
    }

    /**
     * @return mixed
     */
    public function getAuthorImageUrl()
    {
        return $this->authorImageUrl;
    }

    /**
     * @param mixed $authorImage
     */
    public function setAuthorImageUrl($authorImage)
    {
        $this->authorImageUrl = $authorImage;
    }

    /**
     * @return PollutionDataPoint
     */
    public function getPollutionData()
    {
        return $this->pollutionData;
    }

    /**
     * @param mixed $pollutionData
     */
    public function setPollutionData(PollutionDataPoint $pollutionData)
    {
        $this->pollutionData = $pollutionData;
    }

    /**
     * @return string
     */
    public function getExternalUrl()
    {
        return $this->externalUrl;
    }

    /**
     * @param string $externalUrl
     */
    public function setExternalUrl($externalUrl)
    {
        $this->externalUrl = $externalUrl;
    }

    /**
     * @return mixed
     */
    public function getSlug()
    {
        return $this->slug;
    }

    /**
     * @param mixed $slug
     */
    public function setSlug($slug)
    {
        $this->slug = $slug;
    }

    /**
     * @return boolean
     */
    public function isHidden()
    {
        return $this->hidden;
    }

    /**
     * @param boolean $hidden
     */
    public function setHidden($hidden)
    {
        $this->hidden = $hidden;
    }

    /**
     * @return boolean
     */
    public function isThreatened()
    {
        return $this->threatened;
    }

    /**
     * @param boolean $threatened
     */
    public function setThreatened($threatened)
    {
        $this->threatened = $threatened;
    }

    /**
     * @return int
     */
    public function getNumLikes()
    {
        return $this->numLikes;
    }

    /**
     * @param int $numLikes
     */
    public function setNumLikes($numLikes)
    {
        $this->numLikes = $numLikes;
    }

    /**
     * @return int
     */
    public function getNumComments()
    {
        return $this->numComments;
    }

    /**
     * @param int $numComments
     */
    public function setNumComments($numComments)
    {
        $this->numComments = $numComments;
    }

    /**
     * @return string
     */
    public function getThreatenedCaption()
    {
        return $this->threatenedCaption;
    }

    /**
     * @param string $threatenedCaption
     */
    public function setThreatenedCaption($threatenedCaption)
    {
        $this->threatenedCaption = $threatenedCaption;
    }

    /**
     * @return mixed
     */
    public function getTwitterId()
    {
        return $this->twitterId;
    }

    /**
     * @param mixed $twitterId
     */
    public function setTwitterId($twitterId)
    {
        $this->twitterId = $twitterId;
    }

    /**
     * @return mixed
     */
    public function getTwitterStatusUrl()
    {
        return $this->twitterStatusUrl;
    }

    /**
     * @param mixed $twitterStatusUrl
     */
    public function setTwitterStatusUrl($twitterStatusUrl)
    {
        $this->twitterStatusUrl = $twitterStatusUrl;
    }

    /**
     * @return int
     */
    public function getNumRetweets()
    {
        return $this->numRetweets;
    }

    /**
     * @param int $numRetweets
     */
    public function setNumRetweets($numRetweets)
    {
        $this->numRetweets = $numRetweets;
    }

    /**
     * @return mixed
     */
    public function getDefaultIndex()
    {
        return $this->defaultIndex;
    }

    /**
     * @param mixed $defaultIndex
     */
    public function setDefaultIndex($defaultIndex)
    {
        $this->defaultIndex = $defaultIndex;
    }

    /**
     * @return bool
     */
    public function hasEvents(): bool
    {
        return $this->events;
    }

    /**
     * @param bool $events
     */
    public function setEvents(bool $events)
    {
        $this->events = $events;
    }

    /**
     * @return string
     */
    public function getEventsCaption()
    {
        return $this->eventsCaption;
    }

    /**
     * @param string $eventsCaption
     */
    public function setEventsCaption($eventsCaption)
    {
        $this->eventsCaption = $eventsCaption;
    }

    /**
     * @return bool
     */
    public function isVolunteerNeed(): bool
    {
        return $this->volunteerNeed;
    }

    /**
     * @param bool $volunteerNeed
     */
    public function setVolunteerNeed(bool $volunteerNeed)
    {
        $this->volunteerNeed = $volunteerNeed;
    }

    /**
     * @return string
     */
    public function getVolunteerNeedCaption()
    {
        return $this->volunteerNeedCaption;
    }

    /**
     * @param string $volunteerNeedCaption
     */
    public function setVolunteerNeedCaption($volunteerNeedCaption)
    {
        $this->volunteerNeedCaption = $volunteerNeedCaption;
    }

    /**
     * @return bool
     */
    public function hasWalks(): bool
    {
        return $this->walks;
    }

    /**
     * @param bool $walks
     */
    public function setWalks(bool $walks)
    {
        $this->walks = $walks;
    }

    /**
     * @return string
     */
    public function getWalksCaption()
    {
        return $this->walksCaption;
    }

    /**
     * @param string $walksCaption
     */
    public function setWalksCaption($walksCaption)
    {
        $this->walksCaption = $walksCaption;
    }
}
