<?php

namespace TranquilCity\TranquilPavement\Domain\Models;

use App\Mail\PasswordReset;
use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Contracts\Auth\CanResetPassword;
use Illuminate\Support\Facades\Mail;

class User implements Authenticatable, CanResetPassword
{

    private $uid;

    private $email;

    private $passwordHash;

    private $token;

    private $instagramToken;

    private $twitterToken;

    private $twitterTokenSecret;

    private $instagramBusinessId;
    /**
     * @return mixed
     */
    public function getUid()
    {
        return $this->uid;
    }

    /**
     * @param mixed $uid
     */
    public function setUid($uid)
    {
        $this->uid = $uid;
    }

    /**
     * @return mixed
     */
    public function getEmail()
    {
        return $this->email;
    }

    /**
     * @param mixed $email
     */
    public function setEmail($email)
    {
        $this->email = $email;
    }

    /**
     * @return mixed
     */
    public function getPasswordHash()
    {
        return $this->passwordHash;
    }

    /**
     * @param mixed $passwordHash
     */
    public function setPasswordHash($passwordHash)
    {
        $this->passwordHash = $passwordHash;
    }

    /**
     * @return mixed
     */
    public function getToken()
    {
        return $this->token;
    }

    /**
     * @param mixed $token
     */
    public function setToken($token)
    {
        $this->token = $token;
    }

    /**
     * Get the name of the unique identifier for the user.
     *
     * @return string
     */
    public function getAuthIdentifierName()
    {
        return 'uid';
    }

    /**
     * Get the unique identifier for the user.
     *
     * @return mixed
     */
    public function getAuthIdentifier()
    {
        return $this->uid;
    }

    /**
     * Get the password for the user.
     *
     * @return string
     */
    public function getAuthPassword()
    {
        return $this->passwordHash;
    }

    /**
     * Get the token value for the "remember me" session.
     *
     * @return string
     */
    public function getRememberToken()
    {
        return $this->token;
    }

    /**
     * Set the token value for the "remember me" session.
     *
     * @param  string $value
     * @return void
     */
    public function setRememberToken($value)
    {
        $this->token = $value;
    }

    /**
     * Get the column name for the "remember me" token.
     *
     * @return string
     */
    public function getRememberTokenName()
    {
        return 'token';
    }

    /**
     * Return the token from the environment as it is a magic token with all the requirements we need.
     * If we don't have one, fall back to the database value.
     *
     * @return mixed
     */
    public function getInstagramToken()
    {
        return env('INSTAGRAM_API_TOKEN') ?: $this->instagramToken;
    }

    /**
     * @param mixed $instagramToken
     */
    public function setInstagramToken($instagramToken)
    {
        $this->instagramToken = $instagramToken;
    }

    /**
     * Convert the instance to a [ key => value ] array.
     *
     * @return array
     */
    public function toArray()
    {
        $array = get_object_vars($this);
        $array['instagramToken'] = $this->getInstagramToken();
        return $array;
    }

    /**
     * @return string
     */
    public function getTwitterToken()
    {
        return $this->twitterToken;
    }

    /**
     * @param string $twitterToken
     */
    public function setTwitterToken($twitterToken)
    {
        $this->twitterToken = $twitterToken;
    }

    /**
     * @return string
     */
    public function getTwitterTokenSecret()
    {
        return $this->twitterTokenSecret;
    }

    /**
     * @param string $twitterTokenSecret
     */
    public function setTwitterTokenSecret($twitterTokenSecret)
    {
        $this->twitterTokenSecret = $twitterTokenSecret;
    }

    /**
     * Get the e-mail address where password reset links are sent.
     *
     * @return string
     */
    public function getEmailForPasswordReset()
    {
        return $this->email;
    }

    /**
     * Send the password reset notification.
     *
     * @param  string $token
     * @return void
     */
    public function sendPasswordResetNotification($token)
    {
        Mail::to(request()->email)->send(new PasswordReset($token));
    }

    /**
     * @return mixed
     */
    public function getInstagramBusinessId()
    {
        return $this->instagramBusinessId;
    }

    /**
     * @param mixed $instagramBusinessId
     */
    public function setInstagramBusinessId($instagramBusinessId)
    {
        $this->instagramBusinessId = $instagramBusinessId;
    }
}