<?php

namespace TranquilCity\TranquilPavement\Infrastructure\Doctrine\Repositories;

use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Query\ResultSetMapping;
use Doctrine\ORM\Query\ResultSetMappingBuilder;
use TranquilCity\TranquilPavement\Domain\Models\Point;
use TranquilCity\TranquilPavement\Domain\Models\PollutionDataPoint;
use TranquilCity\TranquilPavement\Domain\Repositories\PollutionDataPointRepository;

class DoctrinePollutionDataPointRepository implements PollutionDataPointRepository
{

    private $entityManager;
    private $repository;

    /**
     * Constructs the DoctrineTranquilPlaceRepository
     *
     * @param EntityManager $entityManager The Doctrine Entity Manager (autowired)
     *
     */
    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
        $this->repository = $this->entityManager->getRepository(PollutionDataPoint::class);
    }

    public function add(PollutionDataPoint $point)
    {
        $this->entityManager->persist($point);
    }

    public function saveAll()
    {
        $this->entityManager->flush();
        $this->entityManager->clear();
    }

    public function findAll()
    {
        return $this->repository->findAll();
    }

    public function findById($uid)
    {
        return $this->repository->find($uid);
    }

    public function findByLocation(Point $geolocation)
    {
        $rsm = new ResultSetMapping();
        $rsm->addEntityResult(PollutionDataPoint::class, 'p0_', 'point');
        $rsm->addFieldResult('p0_', 'uid', 'uid');
        $rsm->addFieldResult('p0_', 'geolocation', 'geolocation');
        $rsm->addFieldResult('p0_', 'pm25', 'pm25');
        $rsm->addFieldResult('p0_', 'no2', 'no2');
        $rsm->addFieldResult('p0_', 'arri_db', 'arri_db');
        $rsm->addScalarResult('distance', 'distance');

        $sql = "
          SELECT p0_.*, ST_AsText(p0_.geolocation) as geolocation, ST_DistanceSphere(p0_.geolocation, ST_Point(:x, :y)) as distance
          FROM (
            SELECT * FROM points ORDER BY geolocation <-> ST_SetSRID(ST_Point(:x, :y),4326) LIMIT 10
          ) as p0_ 
          ORDER BY ST_DistanceSphere(p0_.geolocation, ST_Point(:x, :y)) LIMIT 1
        ";

        $parameters = [
            'x' => (string)$geolocation->getLongitude(),
            'y' => (string)$geolocation->getLatitude()
        ];

        $query = $this->entityManager->createNativeQuery(
            $sql,
            $rsm
        );

        $query->setParameters($parameters);

        $result = $query->getSingleResult();

        if ($result['distance'] < 50) {
            return $result['point'];
        }

        return null;

    }

}