<?php

namespace TranquilCity\TranquilPavement\Infrastructure\Services;

use Geocoder\Laravel\ProviderAndDumperAggregator;
use Geocoder\Model\Coordinates;
use TranquilCity\TranquilPavement\Domain\Models\Point;
use TranquilCity\TranquilPavement\Application\Services\Geocoder;

/**
 * Implements the Geocoder domain interface using the Laravel Geocoder library.
 *
 * @category Class
 * @package  TheRestartProject\RepairDirectory\Infrastructure\Services
 * @author   Joaquim d'Souza <joaquim@outlandish.com>
 * @license  http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * @link     http://www.outlandish.com/
 */
class GeocoderImpl implements Geocoder
{

    /**
     * The Laravel Geocoder service
     *
     * @var ProviderAndDumperAggregator
     */
    private $geocoder;

    /**
     * GeocoderImpl constructor.
     *
     * @param ProviderAndDumperAggregator $geocoder The Laravel Geocoder service
     */
    public function __construct($geocoder)
    {
        $this->geocoder = $geocoder;
    }

    /**
     * Find the [lat, lng] of a business
     *
     * @param string $address The address to geolocate
     *
     * @param bool $strict
     * @return Point|null The location of the address, or null if not found
     */
    public function geocode($address, $strict = true)
    {
        $fullAddress = $address;
        if (strpos(strtolower($address), 'london') === false) {
            $fullAddress = $fullAddress . ', London';
        }
        if (strpos(strtolower($address), 'england') === false) {
            $fullAddress = $fullAddress . ', England';
        }
        $geocodeResponse = $this->geocoder->geocode($fullAddress);
        $addressCollection = $geocodeResponse->get();
        $geoAddress = $addressCollection->get(0);
        if ($geoAddress) {
            $fullAddress = $geoAddress->getFormattedAddress();
            if ($strict) {
                $containsAddress = strpos(strtolower($fullAddress), strtolower($address)) !== false;
                $containsLondon = strpos(strtolower($fullAddress), strtolower('London')) !== false;
                if (!$fullAddress || (!$containsAddress && !$containsLondon)) {
                    return null;
                }
            }
            /** @var Coordinates $coordinates */
            $coordinates = $geoAddress->getCoordinates();
            return new Point($coordinates->getLatitude(), $coordinates->getLongitude());
        }
        return null;
    }
}
