<?php
/**
 * Created by PhpStorm.
 * User: Joaquim
 * Date: 12/12/2017
 * Time: 23:30
 */

namespace TranquilCity\TranquilPavement\Infrastructure\Services;


class Instagram extends \MetzWeb\Instagram\Instagram
{

    /**
     * Available scopes
     *
     * @var array
     */
    private $_scopes = array('basic', 'likes', 'comments', 'relationships', 'public_content');

    /**
     * Generates the OAuth login URL
     *
     * @param array $scope
     * @return string Instagram OAuth login URL
     * @throws \Exception
     */
    public function getLoginUrl($scope = array('basic')) {
        if (is_array($scope) && count(array_intersect($scope, $this->_scopes)) === count($scope)) {
            return self::API_OAUTH_URL . '?client_id=' . $this->getApiKey() . '&redirect_uri=' . urlencode($this->getApiCallback()) . '&scope=' . implode('+', $scope) . '&response_type=code';
        } else {
            throw new \Exception("Error: getLoginUrl() - The parameter isn't an array or invalid scope permissions used.");
        }
    }

    /**
     * Get media by its id
     *
     * @param string $shortCode                   Instagram media shortcode
     * @return mixed
     */
    public function getMediaByShortcode($shortCode) {
        return $this->_makeCall('media/shortcode/' . $shortCode, true);
    }

    /**
     * Get a recently tagged media
     *
     * @param string $name                  Valid tag name
     * @param integer [optional] $limit     Limit of returned results
     * @return mixed
     */
    public function getTagMedia($name, $limit = 0) {
        return $this->_makeCall('tags/' . $name . '/media/recent', true, array('count' => $limit));
    }

    public function getTagMediaPaginated($name, $limit = 0, $maxTagId = null) {
        $args = ['count' => $limit];
        if ($maxTagId) {
            $args['max_tag_id'] = $maxTagId;
        }
        return $this->_makeCall('tags/' . $name . '/media/recent', true, $args);
    }
}