<?php

namespace TranquilCity\TranquilPavement\Infrastructure\Services;

use Abraham\TwitterOAuth\TwitterOAuth;
use Illuminate\Http\Request;

class Twitter
{

    private $apiKey;
    private $apiSecret;
    private $callbackUrl;

    /** @var TwitterOAuth */
    private $connection;

    public function __construct($args)
    {
        $this->apiKey = $args['apiKey'];
        $this->apiSecret = $args['apiSecret'];
        $this->callbackUrl = $args['apiCallback'];
    }

    /**
     * Prepare this Twitter service for making API calls
     *
     * @param $accessToken
     * @param $accessTokenSecret
     */
    public function setAccessToken($accessToken, $accessTokenSecret)
    {
        $this->connection = new TwitterOAuth($this->apiKey, $this->apiSecret, $accessToken, $accessTokenSecret);
    }

    /**
     * @param Request $request
     * @return string
     * @throws \Abraham\TwitterOAuth\TwitterOAuthException
     */
    public function getLoginUrl(Request $request)
    {
        $connection = new TwitterOAuth($this->apiKey, $this->apiSecret);
        $requestToken = $connection->oauth('oauth/request_token', array('oauth_callback' => $this->callbackUrl));

        $request->session()->put('twitter_oauth_token', $requestToken['oauth_token']);
        $request->session()->put('twitter_oauth_token_secret', $requestToken['oauth_token_secret']);

        return $connection->url('oauth/authorize', array('oauth_token' => $requestToken['oauth_token']));
    }

    /**
     * @param Request $request
     * @return array
     * @throws \Exception
     */
    public function getOAuthToken(Request $request)
    {
        $requestToken = [];
        $requestToken['oauth_token'] = $request->session()->get('twitter_oauth_token');
        $requestToken['oauth_token_secret'] = $request->session()->get('twitter_oauth_token_secret');

        if ($request->input('oauth_token') !== $requestToken['oauth_token']) {
            throw new \Exception('Error: oauth_token does not match');
        }

        $connection = new TwitterOAuth($this->apiKey, $this->apiSecret, $requestToken['oauth_token'], $requestToken['oauth_token_secret']);
        return $connection->oauth("oauth/access_token", ["oauth_verifier" => $request->input('oauth_verifier')]);
    }

    /**
     * @param $tag
     * @param $limit
     * @param $maxId
     * @return object
     *
     * @throws \Exception
     */
    public function getTagMediaPaginated($tag, $limit, $maxId)
    {
        if (!$this->connection) {
            throw new \Exception('No access token found. Call setAccessToken before making API requests');
        }
        $parameters = [
            'q' => "#${tag}",
            'count' => $limit,
            'include_entities' => true
        ];
        if ($maxId) {
            $parameters['max_id'] = $maxId;
        }
        return $this->connection->get("search/tweets", $parameters);
    }

    /**
     * @param $id
     * @return object
     * @throws \Exception
     */
    public function getTweetById($id)
    {
        if (!$this->connection) {
            throw new \Exception('No access token found. Call setAccessToken before making API requests');
        }
        return $this->connection->get(
            "statuses/show/" . $id,
            [
                'tweet_mode' => 'extended',
                'include_entities' => 'true'
            ]
        );
    }

}